library ieee;
use ieee.std_logic_1164.all;
-- use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;


entity testrecfunc is
  
  port (
    clk : in std_logic;
    abus : in  std_logic_vector(1 downto 0);
    din11 : in  std_logic_vector(7 downto 0);
    din12 : in  std_logic_vector(7 downto 0);
    din13 : in  std_logic_vector(7 downto 0);
    dout1 : out std_logic_vector(7 downto 0);
    din21 : in  std_logic_vector(7 downto 0);
    din22 : in  std_logic_vector(7 downto 0);
    din23 : in  std_logic_vector(7 downto 0);
    dout2 : out std_logic_vector(7 downto 0));

end testrecfunc;


architecture twoproc of testrecfunc is


  constant DWIDTH : integer := 7;      -- data bus width limit
  constant AWIDTH : integer := 2;      -- address bus width limit
  constant MAXADS : integer := 3;       -- max address count plus one
  constant MAXMUX : integer := 2;       -- max number of muxes

  type testrec is record                -- MUX data type
                    datain1 : std_logic_vector(DWIDTH downto 0);  -- input bus #1
                    datain2 : std_logic_vector(DWIDTH downto 0);  -- input bus #2
                    datain3 : std_logic_vector(DWIDTH downto 0);  -- input bus #3
                    dataout : std_logic_vector(DWIDTH downto 0);  -- output bus

                    address : integer range MAXADS downto 0;
                                        -- address bus
                  end record;

  type testrecs is array (natural range <>) of testrec;  -- array of muxes



  -- purpose: update muxes
  procedure updatemuxes (
    muxes : inout testrecs; rawdata : in std_logic_vector(7 downto 0)) is       -- muxes
  begin  -- updatemuxes
    for i in (MAXMUX-1) downto 0 loop
      case muxes(i).address is
        when 0 =>
          muxes(i).dataout := muxes(i).datain1;
        when 1 =>
          muxes(i).dataout := muxes(i).datain2;
        when 2 =>
          muxes(i).dataout := muxes(i).datain3;
        when others =>
          muxes(i).dataout := rawdata;
      end case;
    end loop;  -- i
--  end updatemuxes;
  end;
    

  signal m, nm, pm : testrecs(MAXMUX-1 downto 0);          -- muxes: current, previous, next




    
begin  -- twoproc

  -- purpose: compute new mux values
  -- type   : combinational
  -- inputs : m
  -- outputs: nm
  comb: process (m)
    variable vm : testrecs(MAXMUX-1 downto 0);          -- muxes - new values
  begin  -- process comb
    vm := m;
    updatemuxes(vm,X"5A");
    nm <= vm;  
  end process comb;

  
  -- purpose: update muxes
  -- type   : sequential
  -- inputs : clk, nm
  -- outputs: m
  pupdatemux: process (clk)
  begin  -- process pupdatemux
    if clk'event and clk = '1' then  -- rising clock edge
      m(0).dataout <= nm(0).dataout;
      m(1).dataout <= nm(1).dataout;
    end if;
  end process pupdatemux;


  -- purpose: update the internal data structure
  -- type   : combinational
  -- inputs : datain11, datain12, datain13, datain11, datain12, datain13, address
  -- outputs: m, dataout
  pupdinputs: process (din11, din12, din13, din21, din22, din23, abus)
  begin  -- process pupdinputs
      m(0).datain1 <= din11;
      m(0).datain2 <= din12;
      m(0).datain3 <= din13;
      m(0).address <= conv_integer(abus);
      
      m(1).datain1 <= din21;
      m(1).datain2 <= din22;
      m(1).datain3 <= din23;
      m(1).address <= conv_integer(abus);
      
      dout1 <= m(0).dataout;
      dout2 <= m(1).dataout;
  end process pupdinputs;
  

end twoproc;

